package com.aselimov.app.utilities;

import java.util.List;

public class Statistics {
    /**
     * Calculate the mean of a set of data
     * 
     * @param data     input data
     * @param startIdx start idx of data
     * @param endIdx   end idx of data
     * @return
     */
    public static double calcMean(List<Double> data, int startIdx, int endIdx) {
        double sum = 0.0;
        for (int i = startIdx; i < endIdx; i++) {
            sum += data.get(i);
        }
        return sum / (endIdx - startIdx);
    }

    /**
     * Calculate the variance of a set of data
     * 
     * @param data     input data
     * @param mean     mean value of data
     * @param startIdx start_idx of data
     * @param endIdx   end_idx of data
     * @return variance of data
     */
    public static double calcVariance(List<Double> data, double mean, int startIdx, int endIdx) {
        double variance = 0.0;
        for (int i = startIdx; i < endIdx; i++) {
            double xMinusMean = data.get(i) - mean;
            variance += xMinusMean * xMinusMean;
        }
        return variance / (endIdx - startIdx);
    }

    /**
     * Calculate the covariance of two variables
     * 
     * @param x        List of data representing one variable
     * @param y        List of data representing other variable
     * @param xMean    mean of x data
     * @param yMean    mean of y data
     * @param startIdx start index for both data
     * @param endIdx   end index for both data
     * @return covariance of x and y
     */
    public static double calcCovariance(List<Double> x, List<Double> y, double xMean, double yMean, int startIdx,
            int endIdx) {
        double covariance = 0;
        for (int i = startIdx; i < endIdx; i++) {
            covariance += (x.get(i) - xMean) * (y.get(i) - yMean);
        }
        return covariance / (endIdx - startIdx - 1);
    }

}
