package com.aselimov.app.utilities;

import static org.junit.jupiter.api.Assertions.assertEquals;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

/**
 * Unit test the statistics class
 */
public class StatisticsTest {
    List<Double> x;
    List<Double> y;
    double xMeanFull;
    double xMeanHalf;
    double yMeanFull;
    double yMeanHalf;

    @BeforeEach
    void setup() {
        x = new ArrayList(Arrays.asList(1.30656191, -0.9354737, -1.10250799, 1.00903942, 1.37715207, -0.163614,
                -1.17995963, 0.52334468, -0.24819095, 0.22252544));
        y = new ArrayList(Arrays.asList(0.56089952, 1.9944321, -0.39985776, 0.60433577, 1.01884808,
                0.39328237, 1.72282243, -1.3340576, -0.12969627, -0.1063279));

        xMeanFull = Statistics.calcMean(x, 0, x.size());
        xMeanHalf = Statistics.calcMean(x, 5, x.size());
        yMeanFull = Statistics.calcMean(y, 0, y.size());
        yMeanHalf = Statistics.calcMean(y, 5, y.size());
    }

    @Test
    void testMean() {
        assertEquals(0.08088772442417858, xMeanFull, 1e-8);
        assertEquals(0.43246807465934944, yMeanFull, 1e-8);
        assertEquals(-0.1691788946337382, xMeanHalf, 1e-8);
        assertEquals(0.10920460800791403, yMeanHalf, 1e-8);
    }

    @Test
    void testVariance() {
        assertEquals(0.8451099344901358, Statistics.calcVariance(x, xMeanFull, 0, x.size()), 1e-8);
        assertEquals(0.33219455205301196, Statistics.calcVariance(x, xMeanHalf, 5, x.size()), 1e-8);
        assertEquals(0.8915865222994588, Statistics.calcVariance(y, yMeanFull, 0, y.size()), 1e-8);
        assertEquals(0.9741992662607654, Statistics.calcVariance(y, yMeanHalf, 5, y.size()), 1e-8);
    }

    @Test
    void testCovariance() {
        assertEquals(-0.20175547500418511, Statistics.calcCovariance(x, y, xMeanFull, yMeanFull, 0, x.size()), 1e-8);
        assertEquals(-0.6736187544740592, Statistics.calcCovariance(x, y, xMeanHalf, yMeanHalf, 5, x.size()), 1e-8);
    }

}
